' code is Copyright (c) 2006 Julian M Bucknall, www.boyet.com
Public Class BinarySearchTree(Of K As IComparable(Of K), V)

    Private _root As BSTreeNode(Of K, V)
    Public Property Root() As BSTreeNode(Of K, V)
        Get
            Return _root
        End Get
        Set(ByVal Value As BSTreeNode(Of K, V))
            _root = Value
        End Set
    End Property

    Public Function Contains(ByVal key As K) As Boolean
        Return GetNode(key) IsNot Nothing
    End Function
    Public Function GetNode(ByVal key As K) As BSTreeNode(Of K, V)
        Dim walker As BSTreeNode(Of K, V) = _root

        While walker IsNot Nothing
            Dim nextNode As BSTreeNode(Of K, V) = _
                DirectCast(walker.Advance(key), BSTreeNode(Of K, V))
            If Object.ReferenceEquals(nextNode, walker) Then
                Return nextNode
            End If
            walker = nextNode
        End While
        Return Nothing
    End Function

    Public Function TryGetValue(ByVal key As K, ByRef value As V) As Boolean
        Dim node As BSTreeNode(Of K, V) = GetNode(key)
        If node IsNot Nothing Then
            value = Nothing
            Return False
        End If
        value = node.Value
        Return True
    End Function

    Public Sub Insert(ByVal key As K, ByVal value As V)
        Dim parent As BSTreeNode(Of K, V) = Nothing

        Dim walker As BSTreeNode(Of K, V) = _root

        While walker IsNot Nothing
            Dim nextNode As BSTreeNode(Of K, V) = _
                DirectCast(walker.Advance(key), BSTreeNode(Of K, V))
            If Object.ReferenceEquals(nextNode, walker) Then
                Throw New Exception("Binary Search Tree: inserting duplicate key")
            End If
            parent = walker
            walker = nextNode
        End While

        walker = New BSTreeNode(Of K, V)(key, value)
        If parent Is Nothing Then
            _root = walker
        ElseIf key.CompareTo(parent.Key) < 0 Then
            parent.Left = walker
        Else
            parent.Right = walker
        End If
    End Sub


    Default Public Property ValueByKey(ByVal key As K) As V
        Get
            Dim node As BSTreeNode(Of K, V) = GetNode(key)
            If node Is Nothing Then
                Throw New Exception("Binary Search Tree: Key not found using indexer")
            End If
            Return node.Value
        End Get
        Set(ByVal value As V)
            Dim node As BSTreeNode(Of K, V) = GetNode(key)
            If node Is Nothing Then
                Insert(key, value)
            Else
                node.Value = value
            End If
        End Set
    End Property
End Class