' code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
Imports System.Drawing.Drawing2D

Public Class SimpleNodePainter
    Implements IBinaryNodePainter
    Implements IDisposable

    Protected _surface As Graphics
    Protected _edgePen As Pen
    Protected _externalNodeBrush As Brush
    Protected _borderPen As Pen
    Protected _font As Font
    Protected _fontBrush As Brush

    Public Sub New(ByVal g As Graphics, ByVal unit As Single)
        _surface = g
        _edgePen = New Pen(Color.Black, 2.0F / unit)
        _externalNodeBrush = New SolidBrush(Color.Black)
        _borderPen = New Pen(Color.Black, 2.0F / unit)
        _font = New Font("Consolas", 0.5F)
        _fontBrush = New SolidBrush(Color.Black)
    End Sub

    Public Sub DrawEdge(ByVal childNumber As Integer, ByVal childLevel As Integer, ByVal parentNumber As Integer) Implements IBinaryNodePainter.DrawEdge
        Dim parentY As Integer = childLevel - 1
        _surface.DrawLine(_edgePen, _
                       New PointF(childNumber, childLevel), _
                       New PointF(parentNumber, parentY) _
                       )
    End Sub

    Public Sub DrawEdgedNode(ByVal node As IBinaryNode, ByVal childNumber As Integer, ByVal childLevel As Integer, ByVal parentNumber As Integer) Implements IBinaryNodePainter.DrawEdgedNode
        DrawEdge(childNumber, childLevel, parentNumber)
        DrawNode(node, childNumber, childLevel)
    End Sub

    Protected Overridable Sub DrawData(ByVal node As IBinaryNode, ByVal number As Integer, ByVal nodeRect As RectangleF)
        nodeRect.Offset(0.2F, 0.15F)
        _surface.DrawString(number.ToString(), _font, _fontBrush, nodeRect)
    End Sub
    Public Sub DrawExternalNodes(ByVal node As IBinaryNode, ByVal number As Integer, ByVal level As Integer)
        If node.Left Is Nothing Then
            _surface.DrawLine(_edgePen, _
                              New PointF(number - 0.5F, level + 0.5F), _
                              New PointF(number, level))
            _surface.FillRectangle(_externalNodeBrush, number - 0.8F, level + 0.5F, 0.3F, 0.3F)
        End If
        If node.Right Is Nothing Then
            _surface.DrawLine(_edgePen, _
                              New PointF(number + 0.5F, level + 0.5F), _
                              New PointF(number, level))
            _surface.FillRectangle(_externalNodeBrush, number + 0.5F, level + 0.5F, 0.3F, 0.3F)
        End If
    End Sub
    Public Sub DrawNode(ByVal node As IBinaryNode, ByVal number As Integer, ByVal level As Integer) Implements IBinaryNodePainter.DrawNode
        Dim nodeRect = New RectangleF(number - 0.5F, level - 0.5F, 1.0F, 1.0F)

        Using nodeFill As Brush = New LinearGradientBrush(nodeRect, Color.AliceBlue, Color.SkyBlue, LinearGradientMode.ForwardDiagonal)
            _surface.FillEllipse(nodeFill, nodeRect)
        End Using

        _surface.DrawEllipse(_borderPen, nodeRect)
        DrawData(node, number, nodeRect)
    End Sub

    Private disposedValue As Boolean = False        ' To detect redundant calls

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                If _edgePen IsNot Nothing Then _edgePen.Dispose()
                If _externalNodeBrush IsNot Nothing Then _externalNodeBrush.Dispose()
                If _borderPen IsNot Nothing Then _borderPen.Dispose()
                If _font IsNot Nothing Then _font.Dispose()
                If _fontBrush IsNot Nothing Then _fontBrush.Dispose()
            End If
        End If
        Me.disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

End Class
